@@ -6,11 +6,13 @@
     SPDX-FileCopyrightText: 2008 Friedrich W. H. Kossebau <kossebau@kde.org>
     SPDX-FileCopyrightText: 2010 Teo Mrnjavac <teo@kde.org>
     SPDX-FileCopyrightText: 2017 Harald Sitter <sitter@kde.org>
+    SPDX-FileCopyrightText: 2021 Julius Künzel <jk.kdedev@smartlab.uber.space>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kaboutdata.h"
+#include "kjsonutils.h"
 #include "kpluginmetadata.h"
 
 #include <QCommandLineOption>
@@ -92,8 +94,8 @@ KAboutPerson &KAboutPerson::operator=(const KAboutPerson &other) = default;
 
 KAboutPerson KAboutPerson::fromJSON(const QJsonObject &obj)
 {
-    const QString name = KPluginMetaData::readTranslatedString(obj, QStringLiteral("Name"));
-    const QString task = KPluginMetaData::readTranslatedString(obj, QStringLiteral("Task"));
+    const QString name = KJsonUtils::readTranslatedString(obj, QStringLiteral("Name"));
+    const QString task = KJsonUtils::readTranslatedString(obj, QStringLiteral("Task"));
     const QString email = obj[QStringLiteral("Email")].toString();
     const QString website = obj[QStringLiteral("Website")].toString();
     const QString userName = obj[QStringLiteral("UserName")].toString();
@@ -267,7 +269,7 @@ QString KAboutLicense::text() const
     }
 
     if (knownLicense) {
-        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("kf5/licenses/") + pathToFile);
+        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kf" QT_STRINGIFY(QT_VERSION_MAJOR) "/licenses/") + pathToFile);
         result += QCoreApplication::translate("KAboutLicense", "This program is distributed under the terms of the %1.").arg(name(KAboutLicense::ShortName));
         if (!pathToFile.isEmpty()) {
             result += lineFeed;
@@ -401,6 +403,76 @@ KAboutLicense KAboutLicense::byKeyword(const QString &rawKeyword)
     return KAboutLicense(license, restriction, nullptr);
 }
 
+class KAboutComponentPrivate : public QSharedData
+{
+public:
+    QString _name;
+    QString _description;
+    QString _version;
+    QString _webAddress;
+    KAboutLicense _license;
+};
+
+KAboutComponent::KAboutComponent(const QString &_name,
+                                 const QString &_description,
+                                 const QString &_version,
+                                 const QString &_webAddress,
+                                 enum KAboutLicense::LicenseKey licenseType)
+    : d(new KAboutComponentPrivate)
+{
+    d->_name = _name;
+    d->_description = _description;
+    d->_version = _version;
+    d->_webAddress = _webAddress;
+    d->_license = KAboutLicense(licenseType, nullptr);
+}
+
+KAboutComponent::KAboutComponent(const QString &_name,
+                                 const QString &_description,
+                                 const QString &_version,
+                                 const QString &_webAddress,
+                                 const QString &pathToLicenseFile)
+    : d(new KAboutComponentPrivate)
+{
+    d->_name = _name;
+    d->_description = _description;
+    d->_version = _version;
+    d->_webAddress = _webAddress;
+    d->_license = KAboutLicense();
+    d->_license.setLicenseFromPath(pathToLicenseFile);
+}
+
+KAboutComponent::KAboutComponent(const KAboutComponent &other) = default;
+
+KAboutComponent::~KAboutComponent() = default;
+
+QString KAboutComponent::name() const
+{
+    return d->_name;
+}
+
+QString KAboutComponent::description() const
+{
+    return d->_description;
+}
+
+QString KAboutComponent::version() const
+{
+    return d->_version;
+}
+
+QString KAboutComponent::webAddress() const
+{
+    return d->_webAddress;
+}
+
+KAboutLicense KAboutComponent::license() const
+{
+    return d->_license;
+}
+
+KAboutComponent &KAboutComponent::operator=(const KAboutComponent &other) = default;
+
 class KAboutDataPrivate
 {
 public:
@@ -417,6 +489,7 @@ public:
     QList<KAboutPerson> _authorList;
     QList<KAboutPerson> _creditList;
     QList<KAboutPerson> _translatorList;
+    QList<KAboutComponent> _componentList;
     QList<KAboutLicense> _licenseList;
 #if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 2)
     QString programIconName;
@@ -524,9 +597,7 @@ KAboutData::KAboutData(const KAboutData &other)
     : d(new KAboutDataPrivate)
 {
     *d = *other.d;
-    QList<KAboutLicense>::iterator it = d->_licenseList.begin(), itEnd = d->_licenseList.end();
-    for (; it != itEnd; ++it) {
-        KAboutLicense &al = *it;
+    for (KAboutLicense &al : d->_licenseList) {
         al.d.detach();
         al.d->_aboutData = this;
     }
@@ -536,9 +607,7 @@ KAboutData &KAboutData::operator=(const KAboutData &other)
 {
     if (this != &other) {
         *d = *other.d;
-        QList<KAboutLicense>::iterator it = d->_licenseList.begin(), itEnd = d->_licenseList.end();
-        for (; it != itEnd; ++it) {
-            KAboutLicense &al = *it;
+        for (KAboutLicense &al : d->_licenseList) {
             al.d.detach();
             al.d->_aboutData = this;
         }
@@ -585,6 +654,23 @@ KAboutData &KAboutData::setTranslator(const QString &name, const QString &emailA
     return *this;
 }
 
+KAboutData &KAboutData::addComponent(const QString &name,
+                                     const QString &description,
+                                     const QString &version,
+                                     const QString &webAddress,
+                                     KAboutLicense::LicenseKey licenseKey)
+{
+    d->_componentList.append(KAboutComponent(name, description, version, webAddress, licenseKey));
+    return *this;
+}
+
+KAboutData &
+KAboutData::addComponent(const QString &name, const QString &description, const QString &version, const QString &webAddress, const QString &pathToLicenseFile)
+{
+    d->_componentList.append(KAboutComponent(name, description, version, webAddress, pathToLicenseFile));
+    return *this;
+}
+
 KAboutData &KAboutData::setLicenseText(const QString &licenseText)
 {
     d->_licenseList[0] = KAboutLicense(this);
@@ -842,9 +928,8 @@ QList<KAboutPerson> KAboutData::credits() const
 
 QList<KAboutPerson> KAboutDataPrivate::parseTranslators(const QString &translatorName, const QString &translatorEmail)
 {
-    QList<KAboutPerson> personList;
     if (translatorName.isEmpty() || translatorName == QLatin1String("Your names")) {
-        return personList;
+        return {};
     }
 
     const QStringList nameList(translatorName.split(QLatin1Char(',')));
@@ -854,17 +939,19 @@ QList<KAboutPerson> KAboutDataPrivate::parseTranslators(const QString &translato
         emailList = translatorEmail.split(QLatin1Char(','), Qt::KeepEmptyParts);
     }
 
-    QStringList::const_iterator nit;
-    QStringList::const_iterator eit = emailList.constBegin();
+    QList<KAboutPerson> personList;
+    personList.reserve(nameList.size());
 
-    for (nit = nameList.constBegin(); nit != nameList.constEnd(); ++nit) {
+    auto eit = emailList.constBegin();
+
+    for (const QString &name : nameList) {
         QString email;
         if (eit != emailList.constEnd()) {
             email = *eit;
             ++eit;
         }
 
-        personList.append(KAboutPerson((*nit).trimmed(), email.trimmed(), true));
+        personList.append(KAboutPerson(name.trimmed(), email.trimmed(), true));
     }
 
     return personList;
@@ -890,6 +977,11 @@ QString KAboutData::otherText() const
     return d->_otherText;
 }
 
+QList<KAboutComponent> KAboutData::components() const
+{
+    return d->_componentList;
+}
+
 QList<KAboutLicense> KAboutData::licenses() const
 {
     return d->_licenseList;
@@ -1147,7 +1239,7 @@ void KAboutData::processCommandLine(QCommandLineParser *parser)
                    qPrintable(QCoreApplication::translate("KAboutData CLI", "This application was written by somebody who wants to remain anonymous.")));
         } else {
             printf("%s\n", qPrintable(QCoreApplication::translate("KAboutData CLI", "%1 was written by:").arg(qAppName())));
-            for (const KAboutPerson &person : qAsConst(d->_authorList)) {
+            for (const KAboutPerson &person : std::as_const(d->_authorList)) {
                 QString authorData = QLatin1String("    ") + person.name();
                 if (!person.emailAddress().isEmpty()) {
                     authorData.append(QLatin1String(" <") + person.emailAddress() + QLatin1Char('>'));
@@ -1166,7 +1258,7 @@ void KAboutData::processCommandLine(QCommandLineParser *parser)
         }
     } else if (parser->isSet(QStringLiteral("license"))) {
         foundArgument = true;
-        for (const KAboutLicense &license : qAsConst(d->_licenseList)) {
+        for (const KAboutLicense &license : std::as_const(d->_licenseList)) {
             printf("%s\n", qPrintable(license.text()));
         }
     }
@@ -1186,8 +1278,8 @@ QVariantList listToVariant(const QList<T> &values)
 {
     QVariantList ret;
     ret.reserve(values.count());
-    for (const auto &license : values) {
-        ret << QVariant::fromValue(license);
+    for (const auto &value : values) {
+        ret << QVariant::fromValue(value);
     }
     return ret;
 }
@@ -1211,3 +1303,8 @@ QVariantList KAboutData::translatorsVariant() const
 {
     return listToVariant(d->_translatorList);
 }
+
+QVariantList KAboutData::componentsVariant() const
+{
+    return listToVariant(d->_componentList);
+}
